const strategies = require("./strategies")
const Portfolio = require("./portfolio")

class StrategyManager {

    constructor() {
        this.simulations = null
    }

    init(simulations) {
        this.simulations = simulations
    }

    getOrder(portfolio, predictions, simulation, instrument, options) {
        const _portfolio = new Portfolio(portfolio)

        const currStrategy = strategies[simulation.buyStrategy]

        let ratio = currStrategy.trade(predictions)
        if (options.disableShort || simulation.disableShort) {
            if (ratio < 0) ratio = 0
        }

        const riskAmount = simulation.riskAmount === 0 ? _portfolio.total : simulation.riskAmount
        const buyAmount = ratio * riskAmount

        const position = _portfolio.getPosition(instrument)
        const currAmount = position.value
        const diffAmount = buyAmount - currAmount

        const buyQuantity = diffAmount / position.price

        return { instrument, buyQuantity, diffAmount, lastPrice: position.price }
    }
}

module.exports = new StrategyManager()